/*
 * Copyright (c) 2014, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package com.oracle.java.testlibrary.gc;

import java.lang.management.ManagementFactory;
import java.lang.management.MemoryUsage;
import sun.management.ManagementFactoryHelper;

/**
 * Parent class for all tests of shrinking heap
 * Performs initialize, eating and releasing memory via child's inherited methods 
 * Fails if heap doesn't shrink
 * @author andrey.x.zakharov@oracle.com
 */
public abstract class TestShrinkHeap {

    public static final String MIN_FREE_RATIO_FLAG_NAME = "MinHeapFreeRatio";
    public static final String MAX_FREE_RATIO_FLAG_NAME = "MaxHeapFreeRatio";

    protected TestShrinkHeap() {
    }

    public final void test() {
        System.gc();
        MemoryUsagePrinter.printMemoryUsage("init");

        eat();        
        MemoryUsagePrinter.printMemoryUsage("eaten");
        MemoryUsage muFull = ManagementFactory.getMemoryMXBean().getHeapMemoryUsage();
        
        free();
        MemoryUsagePrinter.printMemoryUsage("free");
        MemoryUsage muFree = ManagementFactory.getMemoryMXBean().getHeapMemoryUsage();

        if (!(muFree.getCommitted() < muFull.getCommitted())) {
            throw new RuntimeException(
                    String.format("committed free heap size is not less than committed full heap size, heap hasn't been shrunk?%n"
                            + "%s = %s%n%s = %s",
                            MIN_FREE_RATIO_FLAG_NAME,
                            ManagementFactoryHelper.getDiagnosticMXBean().getVMOption(MIN_FREE_RATIO_FLAG_NAME).getValue(), 
                            MAX_FREE_RATIO_FLAG_NAME,
                            ManagementFactoryHelper.getDiagnosticMXBean().getVMOption(MAX_FREE_RATIO_FLAG_NAME).getValue()
                    )
            );
        }
    }
    
    protected abstract void eat();
    protected abstract void free();
}
